/* { dg-do assemble { target { powerpc*-*-* && lp64 } } } */
/* { dg-skip-if "do not override -mcpu" { powerpc*-*-* } { "-mcpu=*" } { "-mcpu=power9" } } */
/* { dg-require-effective-target powerpc_p9vector_ok } */
/* { dg-options "-mcpu=power9 -O3" } */

/* Compile with -O3 -mcpu=power9.  It originally generated

        stxsd 12,1(9)

   which is illegal.  */

#pragma pack(1)
struct {
        signed : 1;
        unsigned long a;
} b;

void c(void)
{
        b.a = 0;
        for (; b.a <= 45; b.a = (long)b.a + 1)
                ;
}
